<?php

namespace Objectiv\Plugins\Checkout\Model\Bumps;

class SpecificProductsBump extends BumpAbstract {
	public function is_displayable(): bool {
		$offer_product = $this->get_offer_product();

		if ( ! $this->can_offer_product_be_added_to_the_cart() ) {
			return false;
		}

		if ( $this->quantity_of_product_in_cart( $this->offer_product ) ) {
			return false;
		}

		// Is it a valid upsell (setup correctly) and are there enough units of the offer product to match the cart product?
		// TODO: Technically this disallows upsell offer products that are backordered. Bug or feature? YOU DECIDE.
		if ( $this->is_valid_upsell() && $offer_product->get_manage_stock() && $this->quantity_of_normal_product_in_cart( array_values( $this->products )[0] ) > $offer_product->get_stock_quantity() ) {
			return false;
		}

		$matching_products_in_cart = 0;

		// Count matching products in the cart
		foreach ( $this->products as $product ) {
			if ( $this->quantity_of_normal_product_in_cart( (int) $product ) ) {
				$matching_products_in_cart++;
			}
		}

		// If all products must match and we have fewer products in the cart than in our matching list, return false
		if ( ! $this->any_product && count( $this->products ) > $matching_products_in_cart ) {
			return false;
		}

		// If we get here, matching rule is set to any product, so we can
		// use the number of matching products to determine if we have a match
		return boolval( $matching_products_in_cart );
	}

	public function is_cart_bump_valid(): bool {
		$offer_product = $this->get_offer_product();

		if ( $this->is_valid_upsell() && $this->is_in_cart() ) {
			return true;
		}

		// Is it a valid upsell (setup correctly) and are there enough units of the offer product to match the cart product?
		if ( $this->is_valid_upsell() && $this->quantity_of_product_in_cart( array_values( $this->products )[0] ) > $offer_product->get_stock_quantity() ) {
			return false;
		}

		$matching_products_in_cart = 0;

		// Count matching products in the cart
		foreach ( $this->products as $product ) {
			if ( $this->quantity_of_normal_product_in_cart( (int) $product ) ) {
				$matching_products_in_cart++;
			}
		}

		// If all products must match and we have fewer products in the cart than in our matching list, return false
		if ( ! $this->any_product && count( $this->products ) > $matching_products_in_cart ) {
			return false;
		}

		// If we get here, matching rule is set to any product, so we can
		// use the number of matching products to determine if we have a match
		return boolval( $matching_products_in_cart );
	}
}
