import Main                                       from '../Main';
import DataService                                from './DataService';
import TabService                                 from './TabService';
import ValidationService, { EValidationSections } from './ValidationService';

class OrderReviewStepService {
    constructor() {
        if ( !DataService.getSetting( 'order_review_step_enabled' ) ) {
            return;
        }

        DataService.checkoutForm.on( `cfw_validate_before_tab_switch_${TabService.orderReviewTabId}`, OrderReviewStepService.validateBeforeReviewStep );

        // If submitting fails for any reason, assume a payment issue and go back to the payment tab
        jQuery( document.body ).on( 'cfw_checkout_place_order_event_returned_false', OrderReviewStepService.goToPaymentTabOnError );
        jQuery( document.body ).on( 'cfw_complete_order_failure', OrderReviewStepService.goToPaymentTabOnError );
    }

    static validateBeforeReviewStep(): boolean {
        return ValidationService.validateCustomerInformationTab() && OrderReviewStepService.validatePaymentTab();
    }

    static validatePaymentTab(): boolean {
        // Only validate billing address if billing address is on payment tab
        // And radio toggle is set to use different address
        return !( DataService.getSetting( 'needs_shipping_address' ) && DataService.checkoutForm.find( 'input[name="bill_to_different_address"]:checked' ).val() !== 'same_as_shipping'
            && !ValidationService.validateSection( EValidationSections.BILLING ) );
    }

    static goToPaymentTabOnError(): void {
        const termsCheckbox = jQuery( '#terms' );

        if ( termsCheckbox.length && !termsCheckbox.is( ':checked' ) ) {
            return;
        }

        TabService.go( TabService.paymentMethodTabId );
    }
}

export default OrderReviewStepService;
