import DataService    from './DataService';
import LoggingService from './LoggingService';

class PaymentGatewaysService {
    private _selectedGateway: any = false;

    constructor() {
        // This simulates what WooCommerce core does
        jQuery( document.body ).on( 'click', 'input[name="payment_method"]', () => {
            this.paymentGatewayChangeHandler();
        } );

        jQuery( document.body ).on( 'cfw_pre_updated_checkout', () => {
            this.initSelectedPaymentGateway();
        } );

        jQuery( document.body ).on( 'cfw-payment-tab-loaded', () => {
            // Fix Stripe rendering issue
            setTimeout( () => {
                jQuery( '.wc_payment_method.cfw-active .payment_box' ).hide().show( 0 );
            }, 100 );
        } );

        this.initSelectedPaymentGateway();
    }

    /**
     * Find the selected payment gateway and trigger a click
     *
     * Some gateways look for a click action to init themselves properly
     */
    initSelectedPaymentGateway(): void {
        const paymentMethods = jQuery( '.woocommerce-checkout input[name="payment_method"]' );

        if ( paymentMethods.length === 1 ) {
            paymentMethods.hide();
        }

        // If there was a previously selected method, check that one.
        if ( this._selectedGateway !== false ) {
            jQuery( `#${this._selectedGateway}` ).prop( 'checked', true );
        }

        // If there are none selected, select the first.
        if ( paymentMethods.filter( ':checked' ).length === 0 ) {
            paymentMethods.eq( 0 ).prop( 'checked', true );
        }

        const checkedPaymentMethodId = paymentMethods.filter( ':checked' ).eq( 0 ).prop( 'id' );

        if ( paymentMethods.length > 1 ) {
            // Hide open descriptions.
            jQuery( `div.payment_box:not(".${checkedPaymentMethodId}")` ).filter( ':visible' ).slideUp( 0 );
        }

        paymentMethods.filter( ':checked' ).eq( 0 ).trigger( 'click' );
    }

    paymentGatewayChangeHandler(): void {
        const selectedPaymentMethod = jQuery( '.woocommerce-checkout input[name="payment_method"]:checked' );

        if ( !selectedPaymentMethod.length ) {
            return;
        }

        const placeOrderButton = jQuery( '#place_order' );

        if ( selectedPaymentMethod.data( 'order_button_text' ) ) {
            placeOrderButton.text( selectedPaymentMethod.data( 'order_button_text' ) );
        } else {
            placeOrderButton.text( placeOrderButton.data( 'value' ) );
        }

        const paymentMethod = selectedPaymentMethod.val().toString();

        /** Question: This wc_ajax_url is something we get from the php side, but the function we use has a public filter.
         * So if we use a fake endpoint (%%endpoint%%) when we call it, and someone else is trying to change an ajax endpoint, we won't get the updated value.
         * Becuase their filter will see a $request of %%endpoint%% and rightfully ignore it.
         */
        const url = DataService.getCheckoutParam( 'wc_ajax_url' ).toString().replace( '%%endpoint%%', 'update_payment_method' );
        LoggingService.log( 'Running update_payment_method action. ☄️' );

        const timeInMilliseconds = new Date().getTime();

        jQuery.ajax( {
            type: 'POST',
            url: `${url}&nocache=${timeInMilliseconds}`,
            data: { paymentMethod },
            error: ( xhr: any, textStatus: string, errorThrown: string ) => {
                LoggingService.log( `Update Payment Method Error: ${errorThrown} (${textStatus})` );
            },
            dataType: 'json',
            cache: false,
        } );

        const currentSelectedPaymentMethod = selectedPaymentMethod.attr( 'id' );

        if ( currentSelectedPaymentMethod !== this._selectedGateway ) {
            jQuery( document.body ).trigger( 'payment_method_selected' );
            LoggingService.logEvent( 'Fired payment_method_selected event.' );
        }

        this._selectedGateway = currentSelectedPaymentMethod;
    }
}

export default PaymentGatewaysService;
